/*
    Copyright (C) 2008-2010  Lukas Sommer < SommerLuk at gmail dot com >

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License or (at your option) version 3 or any later version
    accepted by the membership of KDE e.V. (or its successor approved
    by the membership of KDE e.V.), which shall act as a proxy
    defined in Section 14 of version 3 of the license.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QCoreApplication>
#include <QStringList>
#include <QTextStream>

/** Prints the standard help text. */
void syntaxError()
{
  QTextStream console(stdout);
  console << "\nSyntax error.\n\n";
  console << "Use this syntax:\n";
  console << "languagetool [command]\n\n";
  console << "Commands are:\n";
  console << "dump-translation puts out only the translations as simple text file (fuzzy "
                "translations are ignored).\n";
  console << "dump-difference puts out only the entries of the .po file where the translation "
               "differes from the original (empty translations are always ignored).\n";
  console << "simplify-control-sequences simplifies the control sequences that occur in the "
               "text on stdin.\n\n";
  console << "languagetool reads from stdin and writes to stdout.\n\n";
}

/** Reads a block of a po file (one entry with all plural forms, comments...).
    @param stream The stream from which the block is readed.
    @returns The lines of the readed block as string list. */
QStringList readBlock(QTextStream &stream)
{
  QStringList returnValue;
  do {
    returnValue.append(stream.readLine());
  } while (!stream.atEnd() && !returnValue.last().isEmpty());
  return returnValue;
}

/** @param key The key for which we search
    @param block The block in which we search
    @returns The first string in \e block that contains \e key (removing \e key from this string),
    or an empty string of no occurrence is found. */
QString findKey(const QString & key, const QStringList & block)
{
  QString returnValue;
  foreach (const QString & string, block) {
    if (string.startsWith(key)) {
      returnValue = string;
      returnValue.remove(0, key.length());
      break;
    };
  };
  return returnValue;
}

/** Prints to \e stdout and adds a new line. @param block The block to be printed. */
void printBlock (const QStringList & block)
{
  QTextStream console(stdout);
  foreach (const QString & line, block) {
    console << line << '\n';
  };
  console << '\n';
}

/** Processes one entry and writes to stdout only the entries for which the
    translation differs from the original. @param lines The entry to process. */
void processBlock(const QStringList & lines)
{
  if (findKey("msgid_plural ", lines).isEmpty()) { // no plural
    if (findKey("msgid ", lines) != findKey("msgstr ", lines) &&
        findKey("msgstr ", lines) != "\"\"" &&
        findKey("msgid ", lines) != "\"\"") {
      printBlock(lines);
    };
  } else {
    if ((findKey("msgid ", lines) != findKey("msgstr[0] ", lines) ||
         findKey("msgid_plural ", lines) != findKey("msgstr[1] ", lines)) &&
        findKey("msgstr[0] ", lines) != "\"\"" &&
        findKey("msgstr[1] ", lines) != "\"\""){
      printBlock(lines);
    };
  };
}

/** Writes to stdout only the entries for which the translation differs from the original. */
void dumpDifference()
{
  QTextStream stdreader(stdin);
  printBlock(readBlock(stdreader));  // first block is the header
  QString input = stdreader.readAll();  // load the rest of the file
  input.remove("\"\n\"");
  input.remove("\"\r\n\"");
  input.remove("\"\r\"");
  QTextStream inputStream(&input);
  while (!inputStream.atEnd()) {
    processBlock(readBlock(inputStream));
  };
}

/** Reads .po file from stdin and writes to stdout just the
    translations (with the line numbers of the original file). */
void dumpTranslation()
{
  QTextStream stdreader(stdin);
  QTextStream stdwriter(stdout);
  QString actualLine;
  QStringList input;
  while (!stdreader.atEnd()) {
    input.append(stdreader.readLine());
  };
  const int inputSize = input.size();
  bool skipNext = true;  // ignore the header
  bool continueString = false;
  for (int i = 0; i < inputSize; ++i) {
    actualLine = input.at(i);
    if (continueString) {
      if (actualLine.startsWith('"')) {
        stdwriter << actualLine.mid(1, actualLine.length() - 2);
      } else {
        continueString = false;
      };
    };
    if (actualLine.startsWith(QLatin1String("msgstr"))) {
      if (!skipNext) {
        stdwriter << "\n\n(" << i + 1 << ")\n" << actualLine.section('\"', 1, -2);
        continueString = true;
      } else {
        skipNext = false;
      };
    } else if (actualLine.startsWith(QLatin1String("#, fuzzy"))) {
      skipNext = true;
    };
  };
}

/** Simplifies control sequences. */
void simplifyControlSequences()
{
  QString string = QTextStream(stdin).readAll();
  string.replace("&lt;", "<");
  string.replace("&gt;", ">");
  string.replace("\\n", "\n");
  string.replace("\\\"", "\"");
  string.replace("<nl/>", "\n");
  string.replace("%1", "42");  // That's the answer to all questions ;-)
  string.replace("%2", "42");  // That's the answer to all questions ;-)
  string.replace("%3", "42");  // That's the answer to all questions ;-)
  string.replace("%4", "42");  // That's the answer to all questions ;-)
  string.replace("%5", "42");  // That's the answer to all questions ;-)
  string.replace("%6", "42");  // That's the answer to all questions ;-)
  string.replace("%7", "42");  // That's the answer to all questions ;-)
  string.replace("%8", "42");  // That's the answer to all questions ;-)
  string.replace("%9", "42");  // That's the answer to all questions ;-)
  string.remove("<p>");
  string.replace("</p>", "\n");
  string.remove("<em>");
  string.remove("</em>");
  string.remove("<emphasis>");
  string.remove("</emphasis>");
  string.remove("<para>");
  string.replace("</para>", "\n");
  string.remove(QRegExp("<a[A-Za-z01-9 :./=\"]*>"));
  // remove '&' before A-Za-zÄäÖöÜüß
  string.remove(QRegExp("&(?=[A-Za-z\\x00C4\\x00E4\\x00D6\\x00E6\\x00DC\\x00FC\\x00DF])"));
  string.remove("</a>");
  QTextStream(stdout) << string;
}

/** The main function. */
int main (int argc, char * argv[])
{
  QCoreApplication app(argc, argv);
  QStringList arguments = app.arguments();
  if (arguments.value(1) == "dump-translation") {
    dumpTranslation();
  } else if (arguments.value(1) == "dump-difference") {
    dumpDifference();
  } else if (arguments.value(1) == "simplify-control-sequences") {
    simplifyControlSequences();
  } else {
    syntaxError();
  };
  return 0;
}
